/*
 *  Copyright (c) 2000 LSI Logic Corporation.
 *
 *
 *           Name:  MPI_FC.H
 *          Title:  MPI Fibre Channel messages and structures
 *  Creation Date:  June 12, 2000
 *
 *    MPI Version:  01.00.02
 *
 *  Version History
 *  ---------------
 *   File Version:  %version: %
 *   Date Updated:  %date_modified: %
 *     Programmer:  %created_by: %
 *
 *
 *  Date      Version   Description
 *  --------  --------  ------------------------------------------------------
 *  05-08-00  00.10.01  Original release for 0.10 spec dated 4/26/2000.
 *  06-06-00  01.00.01  Update version number for 1.0 release.
 *  06-12-00  01.00.02  Added _MSG_FC_ABORT_REPLY structure.
 *
 *  --------------------------------------------------------------------------
 */

#ifndef MPI_FC_H
#define MPI_FC_H

/*****************************************************************************
*
*        F C    T a r g e t    M o d e    M e s s a g e s
*
*****************************************************************************/

/****************************************************************************/
/* Link Service Buffer Post Request                                         */
/****************************************************************************/
typedef struct _MSG_LINK_SERVICE_BUFFER_POST_REQUEST
{
    U8                      BufferPostFlags;
    U8                      BufferCount;
    U8                      ChainOffset;
    U8                      Function;
    U16                     Reserved;
    U8                      Reserved1;
    U8                      MsgFlags;
    U32                     MsgContext;
    SGE_TRANS_SIMPLE_UNION  SGL;
} MSG_LINK_SERVICE_BUFFER_POST_REQUEST, *PTR_MSG_LINK_SERVICE_BUFFER_POST_REQUEST,
  LinkServiceBufferPostRequest_t, *pLinkServiceBufferPostRequest_t;

#define LINK_SERVICE_BUFFER_POST_FLAGS_PORT_MASK (0x01)

typedef struct _WWNFORMAT
{
    U32                     PortNameHigh;
    U32                     PortNameLow;
    U32                     NodeNameHigh;
    U32                     NodeNameLow;
} WWNFORMAT,
  WwnFormat_t;

/****************************************************************************/
/* Link Service Buffer Post Reply                                           */
/****************************************************************************/
typedef struct _MSG_LINK_SERVICE_BUFFER_POST_REPLY
{
    U16                     Reserved;
    U8                      MsgLength;
    U8                      Function;
    U16                     Reserved1;
    U8                      PortNumber;
    U8                      MsgFlags;
    U32                     MsgContext;
    U16                     Reserved2;
    U16                     IOCStatus;
    U32                     IOCLogInfo;
    U32                     TransferLength;
    U32                     TransactionContext;
    U32                     Rctl_Did;
    U32                     Csctl_Sid;
    U32                     Type_Fctl;
    U16                     SeqCnt;
    U8                      Dfctl;
    U8                      SeqId;
    U16                     Rxid;
    U16                     Oxid;
    U32                     Parameter;
    WWNFORMAT               Wwn;
} MSG_LINK_SERVICE_BUFFER_POST_REPLY, *PTR_MSG_LINK_SERVICE_BUFFER_POST_REPLY,
  LinkServiceBufferPostReply_t, *pLinkServiceBufferPostReply_t;

#define FCP_TARGET_DID_MASK                     (0x00FFFFFF)
#define FCP_TARGET_DID_SHIFT                    (0)
#define FCP_TARGET_RCTL_MASK                    (0xFF000000)
#define FCP_TARGET_RCTL_SHIFT                   (24)
#define FCP_TARGET_SID_MASK                     (0x00FFFFFF)
#define FCP_TARGET_SID_SHIFT                    (0)
#define FCP_TARGET_CSCTL_MASK                   (0xFF000000)
#define FCP_TARGET_CSCTL_SHIFT                  (24)
#define FCP_TARGET_FCTL_MASK                    (0x00FFFFFF)
#define FCP_TARGET_FCTL_SHIFT                   (0)
#define FCP_TARGET_TYPE_MASK                    (0xFF000000)
#define FCP_TARGET_TYPE_SHIFT                   (24)


/****************************************************************************/
/* Link Service Response Request                                            */
/****************************************************************************/
typedef struct _MSG_LINK_SERVICE_RSP_REQUEST
{
    U8                      RspFlags;
    U8                      Reserved;
    U8                      ChainOffset;
    U8                      Function;
    U16                     Reserved1;
    U8                      Reserved2;
    U8                      MsgFlags;
    U32                     MsgContext;
    U32                     Rctl_Did;
    U32                     Csctl_Sid;
    U32                     Type_Fctl;
    U16                     SeqCnt;
    U8                      Dfctl;
    U8                      SeqId;
    U16                     Rxid;
    U16                     Oxid;
    U32                     Parameter;
    SGE_SIMPLE_UNION        SGL;
} MSG_LINK_SERVICE_RSP_REQUEST, *PTR_MSG_LINK_SERVICE_RSP_REQUEST,
  LinkServiceRspRequest_t, *pLinkServiceRspRequest_t;

#define LINK_SERVICE_RSP_FLAGS_IMMEDIATE        (0x80)
#define LINK_SERVICE_RSP_FLAGS_PORT_MASK        (0x01)


/****************************************************************************/
/* Extended Link Service Response Reply                                     */
/****************************************************************************/
typedef struct _MSG_LINK_SERVICE_RSP_REPLY
{
    U16                     Reserved;
    U8                      MsgLength;
    U8                      Function;
    U16                     Reserved1;
    U8                      InitiatorIndex;
    U8                      MsgFlags;
    U32                     MsgContext;
    U16                     Reserved2;
    U16                     IOCStatus;
    U32                     IOCLogInfo;
} MSG_LINK_SERVICE_RSP_REPLY, *PTR_MSG_LINK_SERVICE_RSP_REPLY,
  LinkServiceRspReply_t, *pLinkServiceRspReply_t;


/****************************************************************************/
/* Extended Link Service Send Request                                       */
/****************************************************************************/

typedef struct _MSG_EXLINK_SERVICE_SEND_REQUEST
{
    U8                      SendFlags;
    U8                      Reserved;
    U8                      ChainOffset;
    U8                      Function;
    U32                     MsgFlags_Did;
    U32                     MsgContext;
    U32                     ElsCommandCode;
    SGE_SIMPLE_UNION        SGL;
} MSG_EXLINK_SERVICE_SEND_REQUEST, *PTR_MSG_EXLINK_SERVICE_SEND_REQUEST,
  ExLinkServiceSendRequest_t, *pExLinkServiceSendRequest_t;

#define EX_LINK_SERVICE_SEND_DID_MASK           (0x00FFFFFF)
#define EX_LINK_SERVICE_SEND_DID_SHIFT          (0)
#define EX_LINK_SERVICE_SEND_MSGFLAGS_MASK      (0xFF000000)
#define EX_LINK_SERVICE_SEND_MSGFLAGS_SHIFT     (24)


/****************************************************************************/
/* Extended Link Service Send Reply                                         */
/****************************************************************************/

typedef struct _MSG_EXLINK_SERVICE_SEND_REPLY
{
    U16                     Reserved;
    U8                      MsgLength;
    U8                      Function;
    U16                     Reserved1;
    U8                      Reserved2;
    U8                      MsgFlags;
    U32                     MsgContext;
    U16                     Reserved3;
    U16                     IOCStatus;
    U32                     IOCLogInfo;
    U32                     ResponseLength;
} MSG_EXLINK_SERVICE_SEND_REPLY, *PTR_MSG_EXLINK_SERVICE_SEND_REPLY,
  ExLinkServiceSendReply_t, *pExLinkServiceSendReply_t;

/****************************************************************************/
/* FC Abort Request                                                         */
/****************************************************************************/
typedef struct _MSG_FC_ABORT_REQUEST
{
    U8                      AbortFlags;
    U8                      AbortType;
    U8                      ChainOffset;
    U8                      Function;
    U16                     Reserved1;
    U8                      Reserved2;
    U8                      MsgFlags;
    U32                     MsgContext;
    U32                     TransactionContextToAbort;
} MSG_FC_ABORT_REQUEST, *PTR_MSG_FC_ABORT_REQUEST,
  FcAbortRequest_t, *pFcAbortRequest_t;

#define FC_ABORT_FLAG_PORT_MASK                 (0x01)

#define FC_ABORT_TYPE_ALL_FC_BUFFERS            (0x00)
#define FC_ABORT_TYPE_EXACT_FC_BUFFER           (0x01)

typedef struct _MSG_FC_ABORT_REPLY
{
    U16                     Reserved;
    U8                      MsgLength;
    U8                      Function;
    U16                     Reserved1;
    U8                      Reserved2;
    U8                      MsgFlags;
    U32                     MsgContext;
    U16                     Reserved3;
    U16                     IOCStatus;
    U32                     IOCLogInfo;
} MSG_FC_ABORT_REPLY, *PTR_MSG_FC_ABORT_REPLY,
  FcAbortReply_t, *pFcAbortReply_t;

#endif

